/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
 *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
 *
 *  libmemcached bindings
 *  Copyright (C) 2008 Monty Taylor
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

%nodefaultctor;

class memcached_st {

private:

  memcached_server_st * hosts;
  memcached_result_st result;
  memcached_hash hash;

  void *user_data;
  memcached_clone_func on_clone;
  memcached_cleanup_func on_cleanup;
  memcached_free_function call_free;
  memcached_malloc_function call_malloc;
  memcached_realloc_function call_realloc;
  int send_size;
  int recv_size;
  unsigned int wheel[MEMCACHED_WHEEL_SIZE];
  uint32_t flags;
  int32_t poll_timeout;
  int32_t connect_timeout;
  memcached_allocated is_allocated;
  unsigned int number_of_hosts;
  unsigned int cursor_server;
  int cached_errno;
  memcached_server_distribution distribution;

};

%extend memcached_st {

public:

  memcached_st()
  {
    memcached_st * memc = memcached_create(NULL);
    return memc;
  }

  memcached_st(Memcached * clone)
  {
    WATCHPOINT;
    memcached_st * memc = memcached_clone(NULL, clone);
    WATCHPOINT;
    return memc;
  }

  char *getKey(char *key, size_t key_length, size_t value_length)
  {
    uint32_t flags;
    memcached_return rc;

    return memcached_get($self, key, key_length,
                         &value_length, &flags, &rc);
  }

  char *getByKey(char *master_key, size_t master_length,
                 char *key, size_t key_length,
                 size_t value_length)
  {
    uint32_t flags;
    // TODO: Add a mutex protected static rc here that we can get in an
    //       exception handler
    memcached_return rc;

    return memcached_get_by_key($self, master_key, master_length,
                                key, key_length,
                                &value_length, &flags, &rc);
  }


  %throwexception("MemcachedException") {
    $action
      if (result) { 
      printf("result == %d\n",result);
      if (result != MEMCACHED_SUCCESS) {
        THROW_exception(MemcachedException,"Error: No message yet");
      }
      }
  }

  memcached_return getMultiple(char **keys, size_t *key_length,
                               size_t number_of_keys)
  {

    return memcached_mget($self, keys, key_length, number_of_keys);
  }

  memcached_return setKey(char *key, size_t key_length,
                          char *value, size_t value_length,
                          time_t expiration=0, uint32_t flags=0)
  {
    return memcached_set($self, key, key_length,
                         value, value_length,
                         expiration, flags);
  }

  memcached_return setByKey(char *master_key, size_t master_length,
                            char *key, size_t key_length,
                            char *value, size_t value_length,
                            time_t expiration, uint32_t flags=0 )
  {
    return memcached_set_by_key($self, master_key, master_length,
                                key, key_length,
                                value, value_length,
                                expiration, flags);
  }


  memcached_return add(char *key, size_t key_length,
                       char *value, size_t value_length,
                       time_t expiration=0, uint32_t flags=0)
  {
    printf("setting %s,%d to %s,%d - %d:%d \n",key,key_length,
           value,value_length,expiration,flags);
    return memcached_add($self, key, key_length,
                         value, value_length,
                         expiration, flags);
  }

  memcached_return replaceValue(char *key, size_t key_length,
                                char *value, size_t value_length,
                                time_t expiration=0, uint32_t flags=0)
  {
    return memcached_replace($self, key, key_length,
                             value, value_length,
                             expiration, flags);
  }

  memcached_return prependValue(char *key, size_t key_length,
                                char *value, size_t value_length,
                                time_t expiration=0, uint32_t flags=0)
  {
    return memcached_prepend($self, key, key_length,
                             value, value_length,
                             expiration, flags);
  }


  memcached_return  appendValue(char *key, size_t key_length,
                                char *value, size_t value_length,
                                time_t expiration=0, uint32_t flags=0)
  {
    return memcached_append($self, key, key_length,
                            value, value_length,
                            expiration, flags);
  }
  memcached_return deleteKey(char *key, size_t key_length,
                             time_t expiration=0)
  {
    return memcached_delete($self, key, key_length, expiration);
  }
  memcached_return deleteByKey(char *master_key, size_t master_length,
                               char *key, size_t key_length,
                               time_t expiration=0)
  {
    return memcached_delete_by_key($self, master_key, master_length,
                                   key, key_length, expiration);
  }


  memcached_return incrementKey(char *key, size_t key_length,
                                uint32_t offset, int64_t value) {
    if (value >= 0) {
      uint64_t theVal = (uint64_t)value;
      return memcached_increment($self, key, key_length, offset, &theVal);
    } else {
      return MEMCACHED_NOT_SUPPORTED;
    }

  }
  memcached_return decrementKey(char *key, size_t key_length,
                                uint32_t offset, int64_t value) {
    if (value >= 0) {
      uint64_t theVal = (uint64_t)value;
      return memcached_decrement($self, key, key_length, offset, &theVal);
    } else {
      return MEMCACHED_NOT_SUPPORTED;
    }
  }

  memcached_return flush(time_t expiration) {
    return memcached_flush($self, expiration);
  }

  memcached_return setBehavior(memcached_behavior flag, uint64_t data) {
    return memcached_behavior_set($self, flag, data);
  }

  memcached_return addServer(char * hostname, unsigned int port) {
    return memcached_server_add($self, hostname, port);
  }

  %nothrowexception;


  uint64_t getBehavior(memcached_behavior flag) {
    return memcached_behavior_get($self, flag);
  }

  void quit() {
    memcached_quit($self);
  }

  ~Memcached()
  {
    memcached_free($self);
  }
};

