/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
 *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
 *
 *  libmemcached bindings
 *  Copyright (C) 2008 Monty Taylor
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


/* This tells SWIG to treat char **as a special case when used as a parameter
   in a function call */
%typemap(in) (char **keys, size_t *key_length, size_t number_of_keys) {
  unsigned long i = 0;
  $3 = jenv->GetArrayLength($input);
  $2 = (size_t *) malloc(($3+1)*sizeof(size_t));
  $1 = (char **) malloc(($3+1));
  /* make a copy of each string */
  for (i = 0; i<$3; i++) {
    jstring j_string = (jstring)(jenv->GetObjectArrayElement($input, i));
    const char * c_string = jenv->GetStringUTFChars(j_string, 0);
    size_t the_length = jenv->GetStringUTFLength(j_string);
    $1[i] = (char *)malloc(the_length+1);
    strcpy($1[i], c_string);
    $2[i]=the_length;
    jenv->ReleaseStringUTFChars(j_string, c_string);
    jenv->DeleteLocalRef(j_string);
  }
  $2[i] = 0;
 }

/* This cleans up the memory we malloc'd before the function call */
%typemap(freearg) (char **keys, size_t *key_length,
                   size_t number_of_keys) {
  unsigned long i;
  for (i=0; i<$3-1; i++)
    free($1[i]);
  free($1);
  free($2);
 }

%typemap(jni) char **  "jobjectArray"
%typemap(jtype) char **  "String[]"
%typemap(jstype) char **  "String[]"

 /* This allows a C function to return a char ** as a Java String array */
%typemap(out) char **  {
  unsigned long i;
  unsigned long len=0;
  jstring temp_string;
  const jclass clazz = jenv->FindClass("java/lang/String");

  while ($1[len]) len++;
  jresult = jenv->NewObjectArray(len, clazz, NULL);
  /* exception checking omitted */

  for (i=0; i<len; i++) {
    temp_string = jenv->NewStringUTF(*result++);
    jenv->SetObjectArrayElement(jresult, i, temp_string);
    jenv->DeleteLocalRef(temp_string);
  }
 }

/* These 3 typemaps tell SWIG what JNI and Java types to use */
%typemap(jni) (char **keys, size_t *key_length, size_t number_of_keys) "jobjectArray"
%typemap(jtype) (char **keys, size_t *key_length, size_t number_of_keys) "String[]"
%typemap(jstype)(char **keys, size_t *key_length, size_t number_of_keys) "String[]"

 /* These 2 typemaps handle the conversion of the jtype to jstype typemap type
    and visa versa */
%typemap(javain) (char **keys, size_t *key_length, size_t number_of_keys) "$javainput"
%typemap(javaout) char **  {
  return $jnicall;
 }

